; ION Emulation Shell V1.2 for TSE
; By Robin Kay & Michael Vincent
;
; Copyright 2000, by Michael Vincent & Robin Kay. All rights reserved.
;
.nolist
#include "tse.inc"
.list

.org    userMem-3
.db     $BB,$6D
 ret

 .db "TSE",1           ; Header
title:                 ; Title
 .db " IONEmu V1.3", 0
 .dw 55                ; Stack Required
MenuTitle        .equ title + 1

start:

 ; Initalize Index and Selected
 ld hl, 0
 ld (MenuIndex), hl

rebuildMenu:
 ; Clear screen and display title
 bcall(_clrScrnFull)
 bcall(_cleargbuf)
 bit 5,(iy+18h)
 jr nz, battsareok
 ld ix,BadBattery
 jr showsprite
battsareok:
 ld ix,GoodBattery
showsprite:
 ld b,6
 ld a,88
 ld l,58
 call _tseSprite
 call _tseFastCopy
 ld hl,3900h
 ld (pencol), hl
 ld hl,FreeMem
 bcall(_vputs)
 bcall(_memchk)
 bcall(_setxxxxop2)
 bcall(_op2toop1)
 bcall(_dispop1a)
 ld hl, 0
 ld (pencol), hl
 ld hl, MenuTitle
 bcall(_vputs)
 xor a
 ld (pencol), a
 ld a, (penrow)
 add a, 7
 ld (penrow), a
 ld bc, 0000h+(64-7)
 ld de, 5F00h+(64-7)
 ld h, 1
 bcall(_iline)
 ld c, 07h
 ld e, c
 bcall(_iline)

 ; Init Display menu
 xor a
 ld (MenuLimit), a
 ld a, (MenuIndex)
 ld b, a
loopDrawMenu:

 ; Get program title
 inc b
 push bc
 call getprog
 cp 0
 jr z, notFinished
 pop bc
 jr finished
notFinished:
 call Chkprog
 cp 0
 jr z, notFlash

 ; Mark if flash
Flash:
 push hl
 ld hl, arc
 bcall(_vputs)
 pop hl
notFlash:

 ; Display entry
 bcall(_vputs)
 xor a
 ld (pencol), a
 ld a, (penrow)
 add a, 6
 ld (penrow), a

 ; Increase limit
 ld hl, MenuLimit
 inc (hl)

 ; Check if menu full
 pop bc
 ld a, (hl)
 cp 8
 jr nz, loopDrawMenu
finished:

 ; Handle selected beyond end of menu
 ld a, (MenuSelect)
 ld b, a
 ld a, (MenuLimit)
 dec a
 cp b
 jr nc, skip1
 ld (MenuSelect), a
skip1:

 ; Handle empty menu
 ld a, (MenuLimit)
 cp 0
 jr nz, skip2
 ld a, (MenuIndex)
 cp 0
 jp z, _tseExitShell
 xor a
 ld (MenuIndex), a
 jp rebuildMenu
skip2:

 ; Highlight Entry
 ld a, (MenuSelect)
 call highlightMenuItem

 ; Implement delay
loopMenuKeypad:

 ; Poll keypad
 ei
 halt ; Save power
 bcall(_getCSC)

 ; Handle the quit key
 cp skMode
 call z, _tseExitShell

 ; Handle keys that refresh the display
 ld hl, rebuildMenu
 push hl
 cp skLeft
 jr z, moveLeft
 cp skRight
 jr z, moveRight
 cp skAlpha
 jr z, task_end
 cp sk2nd
 jr z, task_exec
 cp skEnter
 jr z, task_exec
 cp skGraphvar
 jr z, arc_unarc
 pop hl

 ; Handle keys that't don't need to refresh display
 ld hl, loopMenuKeypad
 push hl
 cp skUp
 jr z, moveUp
 cp skDown
 jr z, moveDown
 cp skClear
 jr z, PowerOff

 ; Loop back up
 ret

 ; Move Left
moveLeft:
 ld a, (MenuIndex)
 add a, -8
 ld (MenuIndex), a 
 ret

 ; Move Right
moveRight:
 ld a, (MenuIndex)
 add a, 8
 ld (MenuIndex), a 
 ret

moveUp:

 ; Check if top of list
 ld a, (MenuSelect)
 cp 0
 ret z

 ; Move Up
 call highlightMenuItem
 dec a
 ld (MenuSelect), a
 call highlightMenuItem
 ret

moveDown:

 ; Check if bottom of list
 ld a, (MenuLimit)
 dec a
 ld c, a
 ld a, (MenuSelect)
 cp c
 ret z

 ; Move down
 call highlightMenuItem
 inc a
 ld (MenuSelect), a
 call highlightMenuItem
 ret

 ; Power Off
PowerOff:
 ld a, 1
 out (3),a ; Disable timer?
 ei
 halt ; Wait for ON interrupt
 ret                               

 ; Execute task
task_exec:
 call getsprog
 call _tseChkProg
 cp 3
 jp z, runprogION
 cp 0
 ret nz
 call _tseStartTask
 cp 4
 jr z, ok2
 cp 0
 ret nz
ok2:
 call _tseSwitchTask
 ret

 ; End task
task_end:
 call getsprog
 call _tseChkProg
 cp 0
 ret nz
 call _tseEndTask
 ret

 ; Archive/Unarchive program
arc_unarc:
 call getsprog
 call ChkProg

 ; Borrow system stack
 ld (tempword), sp
 ld hl, (sysstack)
 ld sp, hl

 ; Do arc/unarc and handle errors
 AppOnErr(arc_err)
 bcall(_Arc_UnArc)
 AppOffErr()
arc_err:

 ; Restore stack
 ld hl, (tempword)
 ld sp, hl
 ret

;--Functions--

; Highlight Menu Entry
; a should contain the menu item to be selected
;
highlightMenuItem:
 push af
 ld b, a
 ld a, 7
highlightLoop:
 add a, 6
 djnz highlightLoop  
 ld h, a
 add a, 6
 ld d, a
 ld e, 5Fh
 ld l, 0
 bcall(_invertRect)
 pop af
 ret

; Get number of selected program and runs into getprog
;
getsprog:
 ld a, (MenuSelect)
 ld b, a
 ld a, (MenuIndex)
 add a, b
 ld b, a
 inc b

; Convert TSE prog number into _tseFindVar number
; 
getprog:
 ld c, 0
getprogloop:
 inc c
 ld a, c
 push bc
 call _tseFindVar
 pop bc
 ld a, 0ffh
 ret nz
 push bc
 call ChkProg
 pop bc
 cp 2
 jr z, okFlash
 cp 0
 jr nz, getprogloop
okFlash:
 djnz getprogloop
 xor a
 ret

; Check if program is a valid program of any kind
;
ChkProg:
 call _tseChkProg
 cp 0
 ret z
 cp 2
 ret z

; Check if program is a valid ION program
;
ionChkProg:
 bcall(_ChkFindSym)
 ex de, hl
 ld a, b
 cp 0
 jr z, notIONFlash
 ld de, saferam5
 ld bc, 256
 bcall(_FlashToRam)
 ld hl, saferam5 + 9
 ld c, (hl)
 ld b, 0
 inc bc
 add hl, bc
 call notIONFlash
 cp 0
 ret nz
 ld a, 2
 ret

notIONFlash:
 ld bc, 5
 add hl, bc
 ld a, (hl)
 cp 30h
 ld a, 0ffh
 ret nz
 inc hl
 inc hl
 xor a
 ret

; Run ION program
;
runprogION:

 ; Install loader
 ld hl, ionLoader
 ld de, tempSwapArea
 ld bc, endofldr - ionLoader
 ldir

 ; Install functions
 ld hl, func
 ld de, cmdShadow
 ld bc, endoffunc - func
 ldir

 ; Pre-loader
 ld (tempword), sp
 ld hl, (sysstack)
 ld sp, hl
 bcall(_GrBufClr)
 bcall(_ChkFindSym)
 ld h, d
 ld l, e
 bcall(_ldHLind)
 dec hl
 dec hl
 push hl
 bcall(_EnoughMem)
 pop hl
 jr c, skip
 ld de, userMem

 ; Call loader
 call tempSwapArea

 ; Post-loader
skip:
 ld hl, (tempword)
 ld sp, hl
 ld hl, sysflags
 ld de, 89F0h
 ld bc, 60
 ldir

 ; Restore kernel
 ld hl, tsekrnl
 bcall(_mov9toop1)
 bcall(_chkfindsym)  
 ret c
 ld a, b
 ex de, hl
 ld de, saferam1
 ld bc, 768
 cp 0
 jr z, KrnlRAM
 push bc
 ld bc, 8 + name_length + 5
 add hl, bc
 pop bc
 bcall(_FlashToRam)
 jr KrnlDone
KrnlRAM:
 push de
 ld de, 5
 add hl, de ; Skip header + ret
 pop de
 ldir
KrnlDone:

 ; Return
 ret

; ION Loader
ionLoader:
 push hl
 bcall(_InsertMem)
 bcall(_ChkFindSym)
 ex de, hl
 inc hl
 inc hl
 inc hl
 inc hl
 pop bc
 push bc
 push hl
 ld de, userMem
 ldir
 or a
 call userMem + 1
 pop hl
 pop de
 push de
 ld b, d
 ld c, e
 ex de, hl
 ld hl, userMem
 ldir
 pop de
 ld hl, userMem
 bcall(_DelMem)
 ret
endofldr:

; ION Functions & Jump-Table
func:
jmp5:
 ld de, (pTemp)
 bcall(_CpHLDE)
 ld a, (hl)
 jr nz, dontret
 inc a
 ret
dontret:
 push hl
 and 1fh
 cp 6
 jr nz, jmp1
 dec hl
 dec hl
 dec hl
 ld b, (hl)
 dec hl
 push hl
 ld h, (hl)
 ld l, b
 inc hl
 inc hl
 push ix
 pop de
jmp3:
 ld a, (de)
 or a
 jr z, jmp2
 cp (hl)
 inc de
 inc hl
 jr z, jmp3
 pop hl
jmp1:
 pop hl
 ld bc, 0fffah
 add hl, bc
 ld b, (hl)
 dec hl
jmp4:
 dec hl
 djnz jmp4
 jr jmp5
jmp2:
 ex de, hl
 pop hl
 dec hl
 dec hl
 ld b, (hl) 
 inc b
jmp6:
 dec hl
 djnz jmp6
 ex de, hl
 pop af
 xor a
 ret
ionVersion:
 ld hl, 262 ; v1.6
 xor a
 ld d, a
 ld e, 8
 ret
.org func+80 ; Postion jump-table
 jr ionVersion \ nop
 jp ionRandom
 jp ionPutSprite
 jp ionLargeSprite
 jp ionGetPixel
 jp ionFastCopy
 jr jmp5 \ nop ;ionDetect
 jp ionDecompress
endoffunc:
GoodBattery:
 .db %00001100
 .db %00011110
 .db %00011110
 .db %00011110
 .db %00011110
 .db %00011110
BadBattery:
 .db %00001100
 .db %00011110
 .db %00010010
 .db %00010010
 .db %00010010
 .db %00011110
;--Data
FreeMem:
 .db "Free RAM:  ",0
arc:
.db "(A)",0
tsekrnl:
.db ProtProgObj, "TSEKRNL",0
name_length .equ 9

extdata:

MenuIndex    .equ extdata
MenuSelect   .equ MenuIndex+1
MenuLimit    .equ MenuSelect+1
tempword     .equ MenuLimit+1

.end

